class AlarmDaemonIsNotRunning(Exception):
    ...


class SoundFileNotFound(Exception):
    def __init__(self, sound_path: str):
        self.sound = sound_path

    def __str__(self) -> str:
        return f"Sound file '{self.sound}' was not found."



import os.path
import pickle
import signal
from collections import defaultdict
from datetime import date, datetime, time
from operator import attrgetter
from typing import DefaultDict, List, Optional, Set, Union

from loguru import logger
from prettytable import PrettyTable

from alarmix.schema import RequestAction, TimeMessageSocket, When
from alarmix.utils import DeltaAlarm, add_delta_to_alarms, calculate_auto_time


class AlarmManager:
    """
    AlarmManager manipulates your alarms
    """

    def __init__(self, dump_file: str):
        self.dump_file = dump_file
        self.alarm_pid: Optional[int] = None
        self.alarms: DefaultDict[str, Set[Union[time, datetime]]] = defaultdict(set)

    def process_message(self, msg: TimeMessageSocket) -> str:
        """
        Update alarms by TimeMessageSocket action.
        Delete|Add|List|Stop.
        """
        message = "Something happened"
        if msg.action == RequestAction.delete:
            self.del_alarm(msg.time, msg.when)
            self.dump_alarms()
            message = "Successfully deleted"
        elif msg.action == RequestAction.add:
            self.add_alarm(msg.time, msg.when)
            self.dump_alarms()
            message = "Successfully added"
        elif msg.action == RequestAction.list:
            message = self.list_formatted()
        elif msg.action == RequestAction.stop:
            message = self.stop_alarm()
        return message

    def list_formatted(self) -> str:
        """
        Will return string containing table. Such as:
            +------------+----------------+
            | alarm time | remaining time |
            +------------+----------------+
            |    10:0    |    11:36:06    |
            +------------+----------------+
        """
        table = PrettyTable(field_names=["alarm time", "remaining time"])
        alarms = self.list_alarms()
        for alarm in alarms:
            table.add_row(
                [
                    f"{alarm.time.hour}:{alarm.time.minute}",
                    str(alarm.delta).split(".")[0],
                ]
            )
        return table.get_string()

    def add_alarm(self, event_time: time, when: When) -> None:
        logger.debug(f"Adding {event_time}")
        target = event_time
        if when == When.auto:
            target = calculate_auto_time(event_time)
        self.alarms[when.value].add(target)

    def del_alarm(self, event_time: time, when: When) -> None:
        """
        Delete alarm from queue
        """
        logger.debug(f"Trying delete {event_time}")
        target = event_time
        if when == When.auto:
            target = calculate_auto_time(event_time)
        self.alarms[when.value].discard(target)

    def list_alarms(self) -> List[DeltaAlarm]:
        """
        List alarms sorted by time
        """
        needed_keys = [When.everyday.value]
        if date.today().weekday() < 5:
            needed_keys.append(When.weekdays.value)
        else:
            needed_keys.append(When.weekends.value)
        alarms = set()
        for key in needed_keys:
            alarms.update(self.alarms[key])

        for alarm in self.alarms[When.auto.value]:
            if date.today() == alarm.date():  # type: ignore
                alarms.add(alarm.time())  # type: ignore
        delta_alarms = add_delta_to_alarms(alarms)
        return list(sorted(delta_alarms, key=attrgetter("delta")))

    def stop_alarm(self) -> str:
        """
        Stop alarm by pid.
        Buzzer thread updates alarm_pid variable.
        """
        if self.alarm_pid:
            os.kill(self.alarm_pid, signal.SIGTERM)
            self.alarm_pid = None
            return "Alarm stopped"
        return "Alarm isn't running"

    def cleanup(self) -> None:
        """
        Remove all outdated auto calculated alarms.
        """
        correct_alarms = set()
        for alarm_time in self.alarms[When.auto.value]:
            if alarm_time >= datetime.now():  # type: ignore
                correct_alarms.add(alarm_time)
        self.alarms[When.auto.value] = correct_alarms

    def dump_alarms(self) -> None:
        with open(self.dump_file, "wb") as file:
            pickle.dump(self.alarms, file)

    def load_alarms(self) -> None:
        if not os.path.exists(self.dump_file):
            return None
        with open(self.dump_file, "rb") as file:
            self.alarms = pickle.load(file)



import enum
from datetime import time
from typing import Optional

from pydantic import BaseModel, Field


class When(str, enum.Enum):
    auto = "auto"
    everyday = "everyday"
    weekdays = "weekdays"
    weekends = "weekends"

    def __str__(self) -> str:
        return str(self.value)


class RequestAction(str, enum.Enum):
    add = "add"
    delete = "delete"
    list = "list"
    stop = "stop"


class TimeMessageBase(BaseModel):
    when: When
    action: RequestAction


class TimeMessageClient(TimeMessageBase):
    time: Optional[str]


class TimeMessageSocket(TimeMessageBase):
    time: Optional[time]


class Alarm(BaseModel):
    hours: int
    minutes: int
    when: When = Field(When.auto)



import json
import os.path
import socket
from argparse import ArgumentParser, Namespace
from typing import Optional

from alarmix.exceptions import AlarmDaemonIsNotRunning
from alarmix.schema import RequestAction, TimeMessageClient, When
from alarmix.utils import SOCKET_NAME


def send_message(
    socket_addr: str, time_str: Optional[str], when: str, action: RequestAction
) -> None:
    """
    Communicate with alarm server running on socket-file.
    """
    if not os.path.exists(socket_addr):
        raise AlarmDaemonIsNotRunning()
    msg_obj = TimeMessageClient(time=time_str, when=when, action=action)
    with socket.socket(socket.AF_UNIX, socket.SOCK_STREAM) as sock:
        sock.connect(socket_addr)
        msg = json.dumps(msg_obj.dict()).encode("utf-8")
        sock.sendall(msg)
        data = sock.recv(1024)
        print(data.decode("utf-8"))


def parse_args() -> Namespace:
    arg_parse = ArgumentParser()
    subparsers = arg_parse.add_subparsers(dest="namespace")
    add_parser = subparsers.add_parser("add")
    stop_parser = subparsers.add_parser("stop")
    for parser in (arg_parse, add_parser, stop_parser):
        parser.add_argument(
            "-s",
            "--socket",
            type=str,
            default=SOCKET_NAME,
            help="Socket path to communicate with daemon",
        )
    add_parser.add_argument(
        "time",
        type=str,
        nargs="*",
        help="List of times in format {hours}:{minutes} separated by spaces",
    )
    add_parser.add_argument(
        "-d",
        "--delete",
        dest="delete",
        action="store_true",
        help="If you want to delete this alarms",
    )
    add_parser.add_argument(
        "-w", "--when", default=When.auto, type=When, choices=list(When)
    )
    add_parser.set_defaults(delete=False)
    return arg_parse.parse_args()


def main() -> None:
    args = parse_args()
    try:
        if args.namespace is None:
            send_message(args.socket, None, When.auto, RequestAction.list)
        elif args.namespace == "add":
            for time_str in args.time:
                target_action = RequestAction.add
                if args.delete:
                    target_action = RequestAction.delete
                send_message(args.socket, time_str, args.when, target_action)
        elif args.namespace == "stop":
            send_message(args.socket, None, When.auto, RequestAction.stop)
    except AlarmDaemonIsNotRunning:
        print("Are you sure that timer daemon is running.")


if __name__ == "__main__":
    main()



from argparse import ArgumentParser, Namespace
from pathlib import Path

from loguru import logger

from alarmix.daemon.alarm_manager import AlarmManager
from alarmix.daemon.buzzer import BuzzerThread
from alarmix.daemon.server import ServerThread
from alarmix.exceptions import SoundFileNotFound
from alarmix.utils import SOCKET_NAME


def parse_args() -> Namespace:
    arg_parse = ArgumentParser()
    arg_parse.add_argument("--socket", type=str, default=SOCKET_NAME)
    arg_parse.add_argument(
        "--backup", type=str, default=f"{Path.home()}/.alarms.pickle"
    )
    arg_parse.add_argument("--sound", type=str, default=f"{Path.home()}/alarm.mp3")
    return arg_parse.parse_args()


def run_threads(args: Namespace) -> None:
    local_manager = AlarmManager(args.backup)
    local_manager.load_alarms()
    server = ServerThread(local_manager, args)
    buzzer = BuzzerThread(local_manager, args)
    try:
        server.daemon = True
        buzzer.daemon = True

        server.start()
        buzzer.start()

        server.join()
        buzzer.join()
    finally:
        server.finalize()
        buzzer.finalize()
        logger.info("Goodbye, cowboy")
        raise


def main() -> None:
    args = parse_args()
    try:
        run_threads(args)
    except KeyboardInterrupt:
        print()
        logger.debug("Stopped by keyboard.")
    except SoundFileNotFound as e:
        logger.debug(f"Exception found: {e}")


if __name__ == "__main__":
    main()



import json
import socket
import threading
from argparse import Namespace

from loguru import logger

from alarmix.daemon import lock
from alarmix.daemon.alarm_manager import AlarmManager
from alarmix.schema import TimeMessageSocket
from alarmix.utils import remove_if_exists


class ServerThread(threading.Thread):
    def __init__(self, manager: AlarmManager, args: Namespace):
        threading.Thread.__init__(self)
        self.manager = manager
        self.socket = args.socket

    def finalize(self) -> None:
        remove_if_exists(self.socket)

    def run(self) -> None:
        logger.info("Started daemon")
        remove_if_exists(self.socket)
        server = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        server.bind(self.socket)
        logger.debug(f"Successfully bound {self.socket}")
        while True:
            server.listen(1)
            conn, addr = server.accept()
            msg_str = conn.recv(1024)
            if msg_str:
                params = msg_str.decode("utf-8")
                try:
                    message = TimeMessageSocket(**json.loads(params))
                    lock.acquire()
                    message = self.manager.process_message(message)
                    lock.release()
                    conn.sendall(message.encode("utf-8"))
                except ValueError as err:
                    logger.exception(err)
                    conn.sendall(str(err).encode("utf-8"))
                except Exception as ex:
                    logger.exception(ex)
                    conn.sendall(str(ex).encode("utf-8"))
                conn.close()



import os.path
import subprocess
import threading
from argparse import Namespace
from datetime import datetime
from time import sleep

from loguru import logger

from alarmix.daemon import lock
from alarmix.daemon.alarm_manager import AlarmManager
from alarmix.exceptions import SoundFileNotFound


class BuzzerThread(threading.Thread):
    def __init__(self, manager: AlarmManager, args: Namespace) -> None:
        threading.Thread.__init__(self)
        self.manager = manager
        self.sound = args.sound
        if not os.path.exists(self.sound):
            raise SoundFileNotFound(self.sound)

    def run(self) -> None:
        logger.debug("Started buzzer thread.")
        while True:
            lock.acquire()
            alarms = self.manager.list_alarms()
            self.manager.cleanup()
            lock.release()
            if len(alarms) > 0:
                now = datetime.now().time().replace(second=0, microsecond=0)
                if alarms[0].time == now:
                    lock.acquire()
                    self.manager.alarm_pid = self.start_alarm()
                    lock.release()
            sleep(20)

    def start_alarm(self) -> int:
        process = subprocess.Popen(["mpv", "--loop", "--no-video", self.sound])
        return process.pid

    def finalize(self) -> None:
        lock.acquire()
        self.manager.cleanup()
        self.manager.dump_alarms()
        lock.release()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['alarmix', 'alarmix.client', 'alarmix.daemon']

package_data = \
{'': ['*']}

install_requires = \
['loguru>=0.5.3,<0.6.0', 'prettytable>=1.0.1,<2.0.0', 'pydantic>=1.7.2,<2.0.0']

entry_points = \
{'console_scripts': ['alarmc = alarmix.client.main:main',
                     'alarmd = alarmix.daemon.main:main']}

setup_kwargs = {
    'name': 'alarmix',
    'version': '0.3.13',
    'description': 'Alarm manager server and client',
    'long_description': None,
    'author': 'Pavel Kirilin',
    'author_email': 'win10@list.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import os.path
from dataclasses import dataclass
from datetime import datetime, time, timedelta
from typing import Iterable, List

from loguru import logger

SOCKET_NAME = "/tmp/timer_socket.sock"


@dataclass
class DeltaAlarm:
    time: time
    delta: timedelta


def calculate_auto_time(event_time: time) -> datetime:
    now = datetime.now()
    new_delta = now.replace(hour=event_time.hour, minute=event_time.minute) - now
    target = now + timedelta(seconds=new_delta.seconds)
    target = target.replace(second=0, microsecond=0)
    return target


def add_delta_to_alarms(alarms_list: Iterable[time]) -> List[DeltaAlarm]:
    now = datetime.now()
    alarms_with_delta = []
    for alarm in alarms_list:
        alarm_time = calculate_auto_time(alarm)
        delta = alarm_time - now
        alarms_with_delta.append(DeltaAlarm(time=alarm, delta=delta))
    return alarms_with_delta


def remove_if_exists(filename: str) -> None:
    """
    Removes file
    """
    if os.path.exists(filename):
        logger.debug(f"removing {filename}")
        os.remove(filename)
